#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif


// Matrices are stored column major. In the case of usolve this
// concerns the Jacobian Jf, and for the Jacobian- and Hessian-vector
// products this concerns dx, Jfv, and Hfv, when ndx > 1.

// The user function handle for uopt and usolve.
typedef int (*fhandle)(double const *x, int64_t nx, double *y, int64_t ny, void *context);

// The function handle to compute the gradient for uopt and the
// Jacobian for usolve.
typedef int (*Jhandle)(double const *x, int64_t nx, double *y, int64_t ny, double *Jf, void *context);

// The function handle to compute gradient-vector products for uopt
// and Jacobian-vector products for usolve.
typedef int (*Jvhandle)(double const *x, int64_t nx, double *y, int64_t ny,
                        double const *dx, int64_t ndx, double *Jfv, void *context);

// The function handle to compute vector-Jacobian products for usolve.
typedef int (*Jtvhandle)(double const *x, int64_t nx, double *y, int64_t ny,
                         double const *dy, int64_t ndy, double *Jtv, void *context);

// The function handle to compute the Hessian product for uopt.
typedef int (*Hhandle)(double const *x, int64_t nx, double *y, int64_t ny,
                       double *gf, double *Hf, void *context);

// The function handle to compute Hessian-vector products for uopt.
typedef int (*Hvhandle)(double const *x, int64_t nx, double *y, int64_t ny,
                        double const *dx, int64_t ndx, double *gf, double *Hfv, void *context);

// The function handle to report status.
typedef int (*shandle)(double const *x, int64_t nx, double const *y, int64_t ny,
                       double const *Jf, double const *,
                       char const **snames, char const **svals, int64_t nstats,
                       void *context);


int uopt(double const *x0, double *xs, int64_t nvar, double *ys, int64_t ny,
         fhandle f, Jhandle g, Jvhandle Jv, Hhandle H, Hvhandle Hv, shandle status,
         char const **onames, double *ovals, int64_t nopts,
         void *context);


int usolve(double const *x0, double *xs, int64_t nvar, double *ys, int64_t ny,
           fhandle f, Jhandle J, Jvhandle Jv, Jtvhandle Jtv, void* Hv, shandle status,
           char const **onames, double *ovals, int64_t nopts,
           void *context);

typedef void (*ErrorHandler)(int);
ErrorHandler uoptSetErrorhandler(ErrorHandler newErrorHandler);

#ifdef __cplusplus
}
#endif
