#include <uopt.h>

#include <valarray>
#include <iostream>
#include <string.h>

// find y
// y = sqrt(x);

struct MySettings {
  std::valarray<double> x;
};

int fsqrt_residual(double const *y_, int64_t N, double *res, int64_t, void *context) {
  MySettings *s = (MySettings *)context;
  std::valarray<double> y(y_, N);
  y *= y;
  y -= s->x;
  y *= y;
  *res = sqrt(y.sum());
  return 0;
}

typedef std::valarray<double> Vector;

int gsqrt_residual(double const *y_, int64_t N, double *res, int64_t, double *gy_, void *context) {
  MySettings *s = (MySettings *)context;
  Vector y(y_, N);
  Vector const v1 = y*y;
  Vector const v2 = v1 - s->x;
  Vector const v3 = v2*v2;
  double const v4 = v3.sum();
  *res = sqrt(v4);

  double a_res = 1;

  // double const res = sqrt(v4);
  double a_v4 = a_res * 0.5 / sqrt(v4);

  // double const v4 = v3.sum();
  Vector a_v3(a_v4, v3.size());

  // Vector const v3 = v2*v2;
  Vector a_v2(a_v3 * v2 * 2);

  // Vector const v2 = v1 - s->x;

  // Vector const v1 = y*y;
  Vector a_y(a_v2 * y * 2);

  memcpy(gy_, &a_y[0], sizeof(double)*a_y.size());

  return 0;
}

int Hsqrt_residual(double const *y_, int64_t N, double *res, int64_t, double *gy_, double */*Hy_*/, void *context) {

  MySettings *s = (MySettings *)context;
  Vector const y(y_, N);
  Vector const v1 = y*y;
  Vector const v2 = v1 - s->x;
  Vector const v3 = v2*v2;
  double const v4 = v3.sum();
  *res = sqrt(v4);

  double a_res = 1;

  // double const res = sqrt(v4);
  double a_v4 = a_res * 0.5 / sqrt(v4);

  // double const v4 = v3.sum();
  Vector a_v3(a_v4, v3.size());

  // Vector const v3 = v2*v2;
  Vector a_v2(a_v3 * v2 * 2);

  // Vector const v2 = v1 - s->x;

  // Vector const v1 = y*y;
  Vector a_y(a_v2 * y * 2);

  memcpy(gy_, &a_y[0], sizeof(double)*a_y.size());

  return 0;
}


int main() {

  long N = 7;

  std::valarray<double> x(N);
  x = 17;

  std::valarray<double> x0(N);
  x0 = 1.111;

  std::valarray<double> xs(N);

  MySettings context;
  context.x = x;

  long const nopts = 4;
  char const *options[nopts] = {
                                "sdirs",
                                "trace",
                                "checkHessian",
                                "wolcheck"
  };
  double ovals[nopts] = {
                         2,
                         1,
                         -1,
                         0
  };

  auto uopt_ouput = [](double const */*x*/, int64_t /*nx*/,
                       double const */*y*/, int64_t /*ny*/,
                       double const */*gf*/, double const */*Hf*/,
                       char const **snames, char const **svals, int64_t nstats,
                       void */*context*/) -> int
                    {
                      for (long i = 0; i < nstats; ++i) {
                        if (i > 0) std::cout << ", ";
                        std::cout << snames[i] << "=" << svals[i];
                      }
                      std::cout << "\n";
                      return 0;
                    };

  double ys = 0;
  int rc = uopt(&x0[0], &xs[0], x0.size(), &ys, 1,
                fsqrt_residual, gsqrt_residual, 0, Hsqrt_residual, 0,
                uopt_ouput,
                options, ovals, nopts,
                &context);

  std::cerr << "result: " << rc << "\n";

  std::cout << "sol: \n";
  for (unsigned i = 0; i < xs.size(); ++i) {
    if (i > 0) {
      std::cout << ", ";
    }
    std::cout << xs[i];
  }
  std::cout << "\n";

}
