context("ruopt/run-C_uopt-nargs-check")

library(uoptr)

test_that("C_uoptr raises error when arg 1 is not numeric", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr, list(c(2,2)), c(3,4), 5:6, 7:8, 9:10, 11:12, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises error when arg 2 is not a function", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr, c(2,2), c(3,4), 5:6, 7:8, 9:10, 11:12, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises error when arg 3 is not a function", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr,
                   c(2,2),
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   5:6, 7:8, 9:10, 11:12, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises error when arg 4 is not a function", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr,
                   c(2,2),
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   7:8, 9:10, 11:12, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises error when arg 5 is not a function", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr,
                   c(2,2),
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   9:10, 11:12, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises error when arg 6 is not a function", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr,
                   c(2,2),
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   9:10, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises error when arg 7 is not a list", {
    err <- FALSE
    tryCatch(
        r <- .Call(C_uoptr,
                   c(2,2),
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   function(x) { list(f=2*x, J=array(2, c(1, 3))) },
                   11:12, 11:12, 11:12, 11:12),
        error = function(e) {
            show(e)
            err <<- TRUE
        })
    expect_true(err)
})

test_that("C_uoptr raises no error when all OK", {
    err <- FALSE
    lo <- list(a=11, b=4, 5, g=7, checkgradient=-1, checkHessian=-1, maxit=2)
    tryCatch(
        r <- .Call(C_uoptr,
                   c(1.22,2,5,7,9),
                   function(x) { 2*x[[1]] },
                   function(x) { list(J=array(2*x, c(1, length(x))), f=2*x[[1]]) },
                   function(x) { list(H=matrix(x, length(x), length(x)),
                                      J=array(x, c(1, length(x))),
                                      f=2*x[[1]]
                                      ) },
                   function(x, status) { list(H=matrix(x, length(x), length(x)),
                                      J=array(x, c(1, length(x))),
                                      f=2*x[[1]]
                                      ) },
                   function(x) { list(J=array(x, c(1, length(x))),
                                      f=2*x[[1]]
                                      ) },
                   function(x, y, status) {
                       show(list(output.x=x, output.status=status))
                       0
                   },
                   lo,
                   environment()),
        error = function(e) {
            show(e)
            err <<- TRUE
        })

    expect_true(!err)

    show(r)

})
